@@ -1,11 +1,10 @@
 package Search::OpenSearch::Response::Tiny;
-use strict;
-use warnings;
+use Moose;
 use Carp;
-use base qw( Search::OpenSearch::Response::JSON );
+extends 'Search::OpenSearch::Response::JSON';
 use JSON;
 
-our $VERSION = '0.31';
+our $VERSION = '0.400';
 
 my %tiny_fields = map { $_ => 1 } qw(
     total
